@@ -30,6 +30,11 @@
 #' * `task_properties` (list of `character()`)\cr
 #'   List of vectors of supported [Task] properties, named by their task type.
 #'
+#' * `task_mandatory_properties` (list of `character()`)\cr
+#'   List of vectors of [Task] properties which necessarily must be supported by the [Learner].
+#'   I.e., if the task property is not found in the set of the learner properties, an exception
+#'   is raised.
+#'
 #' * `learner_properties` (list of `character()`)\cr
 #'   List of vectors of supported [Learner] properties, named by their task type.
 #'
@@ -99,8 +104,13 @@ local({
     regr = tmp
   )
 
+  mlr_reflections$task_mandatory_properties = list(
+    classif = c("twoclass", "multiclass")
+  )
+
+
   ### Learner
-  tmp = c("featureless", "missings", "weights", "importance", "selected_features", "oob_error", "loglik")
+  tmp = c("featureless", "missings", "weights", "importance", "selected_features", "oob_error", "loglik", "hotstart_forward", "hotstart_backward")
   mlr_reflections$learner_properties = list(
     classif = c(tmp, "twoclass", "multiclass"),
     regr = tmp
@@ -116,7 +126,7 @@ local({
 
 
   ### Measures
-  tmp = c("na_score", "requires_task", "requires_learner", "requires_train_set")
+  tmp = c("na_score", "requires_task", "requires_learner", "requires_model", "requires_train_set")
   mlr_reflections$measure_properties = list(
     classif = tmp,
     regr = tmp
