@@ -12,6 +12,7 @@
 #' @template param_store_models
 #' @template param_store_backends
 #' @template param_encapsulate
+#' @template param_allow_hotstart
 #' @return [ResampleResult].
 #'
 #' @template section_parallelization
@@ -51,7 +52,7 @@
 #' bmr1 = as_benchmark_result(rr)
 #' bmr2 = as_benchmark_result(rr_featureless)
 #' print(bmr1$combine(bmr2))
-resample = function(task, learner, resampling, store_models = FALSE, store_backends = TRUE, encapsulate = NA_character_) {
+resample = function(task, learner, resampling, store_models = FALSE, store_backends = TRUE, encapsulate = NA_character_, allow_hotstart = FALSE) {
   task = assert_task(as_task(task, clone = TRUE))
   learner = assert_learner(as_learner(learner, clone = TRUE))
   resampling = assert_resampling(as_resampling(resampling))
@@ -72,27 +73,56 @@ resample = function(task, learner, resampling, store_models = FALSE, store_backe
     NULL
   }
 
+  grid = if (allow_hotstart) {
+   hotstart_grid = map_dtr(seq_len(n), function(iteration) {
+      if (!is.null(learner$hotstart_stack)) {
+        # search for hotstart learner
+        task_hashes = task_hashes(task, resampling)
+        start_learner = get_private(learner$hotstart_stack)$.start_learner(learner$clone(), task_hashes[iteration])
+      }
+      if (is.null(learner$hotstart_stack) || is.null(start_learner)) {
+        # no hotstart learners stored or no adaptable model found
+        mode = "train"
+      } else {
+        # hotstart learner found
+        start_learner$param_set$values = insert_named(start_learner$param_set$values, learner$param_set$values)
+        learner = start_learner
+        mode = "hotstart"
+      }
+      data.table(learner = list(learner), mode = mode)
+    })
+    # null hotstart stack to reduce overhead in parallelization
+    map(hotstart_grid$learner, function(learner) {
+      learner$hotstart_stack = NULL
+      learner
+    })
+    hotstart_grid
+  } else {
+    data.table(learner = replicate(n, learner), mode = "train")
+  }
+
   if (getOption("mlr3.debug", FALSE)) {
     lg$info("Running resample() sequentially in debug mode with %i iterations", n)
-
-    res = lapply(seq_len(n), workhorse,
-      task = task, learner = learner, resampling = instance,
-      store_models = store_models, lgr_threshold = lg$threshold, pb = pb
+    res = mapply(workhorse,
+      iteration = seq_len(n), learner = grid$learner, mode = grid$mode,
+      MoreArgs = list(task = task, resampling = instance, store_models = store_models, lgr_threshold = lg$threshold,
+        pb = pb), SIMPLIFY = FALSE
     )
   } else {
     lg$debug("Running resample() via future with %i iterations", n)
 
-    res = future.apply::future_lapply(seq_len(n), workhorse,
-      task = task, learner = learner, resampling = instance,
-      store_models = store_models, lgr_threshold = lg$threshold, pb = pb,
-      future.globals = FALSE, future.scheduling = structure(TRUE, ordering = "random"),
-      future.packages = "mlr3", future.seed = TRUE
+    res = future.apply::future_mapply(workhorse,
+      iteration = seq_len(n), learner = grid$learner, mode = grid$mode,
+      MoreArgs = list(task = task, resampling = instance, store_models = store_models, lgr_threshold = lg$threshold,
+      pb = pb),
+      SIMPLIFY = FALSE, future.globals = FALSE, future.scheduling = structure(TRUE, ordering = "random"),
+      future.packages = "mlr3", future.seed = TRUE, future.stdout = future_stdout()
     )
   }
 
   data = data.table(
     task = list(task),
-    learner = list(learner),
+    learner = grid$learner,
     learner_state = map(res, "learner_state"),
     resampling = list(instance),
     iteration = seq_len(n),
