@@ -9,7 +9,7 @@
 #' @importFrom stats predict rnorm runif sd contr.treatment
 #' @importFrom uuid UUIDgenerate
 #' @importFrom parallelly availableCores
-#' @importFrom future nbrOfWorkers
+#' @importFrom future nbrOfWorkers plan
 #'
 #' @section Learn mlr3:
 #' * Book on mlr3: \url{https://mlr3book.mlr-org.com}
@@ -67,13 +67,14 @@ dummy_import = function() {
   lg = lgr::get_logger(pkgname)
   assign("lg", lg, envir = parent.env(environment()))
   f = function(event) {
-    event$msg = paste0("[mlr3]  ", event$msg)
+    event$msg = paste0("[mlr3] ", event$msg)
     TRUE
   }
-  lg$add_filter(f)
+  lg$set_filters(list(f))
   if (Sys.getenv("IN_PKGDOWN") == "true") {
     lg$set_threshold("warn")
   }
+
 } # nocov end
 
 leanify_package()
