@@ -140,7 +140,17 @@ expect_backend = function(b) {
   checkmate::expect_integerish(b$missings(b$rownames[0L], "_not_existing_"), len = 0L, names = "unique")
 
   # $hash
- checkmate::expect_string(b$hash)
+  checkmate::expect_string(b$hash)
+
+  # col_hashes
+  col_hashes = b$col_hashes
+  checkmate::expect_character(col_hashes, any.missing = FALSE)
+  checkmate::expect_names(names(col_hashes), permutation.of = setdiff(b$colnames, b$primary_key))
+
+  # if multiple cols have the same hash, check that they actually contain the same data.
+  realhashes = sapply(b$data(rows = b$rownames, cols = setdiff(b$colnames, b$primary_key)), mlr3misc::calculate_hash)
+  expect_equal(unname(sapply(split(realhashes, col_hashes), data.table::uniqueN)), rep(1, data.table::uniqueN(col_hashes)))
+
 }
 
 expect_iris_backend = function(b, n_missing = 0L) {
@@ -213,13 +223,14 @@ expect_task = function(task, null_backend_ok = TRUE) {
     checkmate::expect_data_table(task$head(1), nrows = 1L)
   }
 
-  cols = c("id", "type", "levels", "label")
+  cols = c("id", "type", "levels", "label", "fix_factor_levels")
   checkmate::expect_data_table(task$col_info, key = "id", ncols  = length(cols))
   checkmate::expect_names(names(task$col_info), permutation.of = cols)
   expect_id(task$col_info$id)
   checkmate::expect_subset(task$col_info$type, mlr3::mlr_reflections$task_feature_types)
   checkmate::expect_list(task$col_info$levels)
   checkmate::expect_character(task$col_info$label)
+  checkmate::expect_logical(task$col_info$fix_factor_levels)
 
   checkmate::expect_list(task$col_roles, names = "unique", any.missing = FALSE)
   checkmate::expect_names(names(task$col_roles), permutation.of = mlr3::mlr_reflections$task_col_roles[[task$task_type]])
@@ -256,6 +267,18 @@ expect_task = function(task, null_backend_ok = TRUE) {
     data = task$data(rows = task$row_ids[0L], data_format = "data.table")
     checkmate::expect_data_table(data, nrows  = 0L)
   }
+
+  # col_hashes
+  col_hashes = task$col_hashes
+  checkmate::expect_character(col_hashes, any.missing = FALSE)
+  checkmate::expect_names(names(col_hashes), permutation.of = setdiff(unlist(task$col_roles), task$backend$primary_key))
+
+  # if multiple cols have the same hash, check that they actually contain the same data.
+  if (!null_backend) {
+    realhashes = sapply(task$data(cols = names(col_hashes)), mlr3misc::calculate_hash)
+    expect_equal(unname(sapply(split(realhashes, col_hashes), data.table::uniqueN)), rep(1, data.table::uniqueN(col_hashes)))
+  }
+
 }
 
 expect_task_supervised = function(task) {
